@@ -18,10 +18,6 @@
  * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
-#ifndef lint
-static const char rcsid[] =
-    "@(#) $Header: /usr/anzen/src/fragrouter/fragrouter/libpcap-0.4/gencode.c,v 1.1.1.1 1999/05/03 04:06:43 dugsong Exp $ (LBL)";
-#endif
 
 #include <sys/types.h>
 #include <sys/socket.h>
