@@ -20,10 +20,6 @@
  *
  *  Optimization module for tcpdump intermediate representation.
  */
-#ifndef lint
-static const char rcsid[] =
-    "@(#) $Header: /usr/anzen/src/fragrouter/fragrouter/libpcap-0.4/optimize.c,v 1.1.1.1 1999/05/03 04:06:44 dugsong Exp $ (LBL)";
-#endif
 
 #include <sys/types.h>
 #include <sys/time.h>
