@@ -36,10 +36,6 @@
  *      kernel.
  */
 
-#ifndef lint
-static const char rcsid[] =
-    "@(#) $Header: /usr/anzen/src/fragrouter/fragrouter/libpcap-0.4/pcap-dlpi.c,v 1.1.1.1 1999/05/03 04:06:44 dugsong Exp $ (LBL)";
-#endif
 
 #include <sys/types.h>
 #include <sys/time.h>
