@@ -165,11 +165,11 @@ static inline unsigned short int csum_tcpudp_magic(unsigned long saddr,
  * in icmp.c
  */
 
-#if 0
-static inline unsigned short ip_compute_csum(unsigned char * buff, int len) {
+
+inline unsigned short ip_compute_csum(unsigned char * buff, int len) {
     return csum_fold (csum_partial(buff, len, 0));
 }
-#endif
+
 
 __inline__ unsigned short my_tcp_check(struct tcphdr *th, int len,
 	unsigned long saddr, unsigned long daddr)
@@ -220,6 +220,11 @@ unsigned short ip_fast_csum(unsigned short * addr,int len)
 return ip_check_ext(addr,len<<2,0);
 }
 
+unsigned short ip_compute_csum(unsigned short * addr,int len)
+{
+return ip_check_ext(adr,len,0);
+}
+
 struct psuedo_hdr
 {
     long saddr;      
