@@ -219,7 +219,7 @@ ip_frag_create (int offset, int end, struct sk_buff *skb, unsigned char *ptr)
   if (fp == NULL)
     {
 //      NETDEBUG (printk ("IP: frag_create: no memory left !\n"));
-	mem_low();
+	nids_params.no_mem("ip_frag_create");
       return (NULL);
     }
   memset (fp, 0, sizeof (struct ipfrag));
@@ -432,7 +432,7 @@ ip_create (struct ip *iph)
   if (qp == NULL)
     {
 //      NETDEBUG (printk ("IP: create: no memory left !\n"));
- 	mem_low();
+ 	nids_params.no_mem("ip_create");
       return (NULL);
     }
   memset (qp, 0, sizeof (struct ipq));
@@ -446,7 +446,7 @@ ip_create (struct ip *iph)
   if (qp->iph == NULL)
     {
 //      NETDEBUG (printk ("IP: create: no memory left !\n"));
-	mem_low();
+	nids_params.no_mem("ip_create");
       frag_kfree_s (qp, sizeof (struct ipq));
       return (NULL);
     }
@@ -536,7 +536,7 @@ ip_glue (struct ipq *qp)
   if ((skb = (char *) malloc (len)) == NULL)
     {
 //      NETDEBUG (printk ("IP: queue_glue: no memory for gluing queue %p\n", qp));
-	mem_low();
+	nids_params.no_mem("ip_glue");
       ip_free (qp);
       return (NULL);
     }
@@ -785,7 +785,7 @@ nids_params.syslog(NIDS_WARN_IP,NIDS_WARN_IP_OVERLAP,iph,0);
 
   if (!tfp)
     {
-	mem_low();
+	nids_params.no_mem("ip_defrag");
       kfree_skb (skb, FREE_READ);
       return NULL;
     }
@@ -858,7 +858,7 @@ ip_frag_init (int n)
   time0 = tv.tv_sec;
   fragtable = (struct hostfrags **) malloc (n * sizeof (struct hostfrags *));
   if (!fragtable)
-    mem_low ();
+    nids_params.no_mem("ip_frag_init");
   memset (fragtable, 0, n * sizeof (struct hostfrags *));
   hash_size = n;
   return 0;
