@@ -1,7 +1,7 @@
 /*
-Copyright (c) 1999 Rafal Wojtczuk <nergal@icm.edu.pl>. All rights reserved.
-See the file COPYING for license details.
-*/
+   Copyright (c) 1999 Rafal Wojtczuk <nergal@avet.com.pl>. All rights reserved.
+   See the file COPYING for license details.
+ */
 
 #include <pcap.h>
 #include <stdlib.h>
@@ -18,7 +18,7 @@ See the file COPYING for license details.
 
 extern char *inet_ntoa (long);
 
-char errbuf[PCAP_ERRBUF_SIZE];
+char nids_errbuf[PCAP_ERRBUF_SIZE];
 
 static struct proc_node *ip_frag_procs;
 static struct proc_node *ip_procs;
@@ -27,6 +27,7 @@ extern int tcp_init (int);
 extern int ip_frag_init (int);
 extern void scan_init ();
 extern void process_tcp (u_char *, int);
+extern void process_icmp (u_char *);
 extern int ip_defrag_stub (char *, struct ip **);
 extern int ip_options_compile (char *);
 static int linkoffset = 0;
@@ -47,9 +48,18 @@ char *nids_warnings[] =
 };
 
 static void nids_syslog (int, int, struct ip *, void *);
+extern void nids_no_mem (char *);
+static int nids_ip_filter (struct ip *);
 
 struct nids_prm nids_params =
-{1024, 256, 0, 168, -1, nids_syslog, LOG_ALERT, 256, 3000, 10};
+{1024, 256, 0, 168, -1, nids_syslog, LOG_ALERT, 256, 3000, 10, nids_no_mem,
+ nids_ip_filter};
+
+static int 
+nids_ip_filter (struct ip *x)
+{
+  return 1;
+}
 
 static void
 nids_syslog (int type, int errnum, struct ip *iph, void *data)
@@ -133,8 +143,9 @@ pcap_hand (u_char * par, struct pcap_pkthdr *hdr, u_char * data)
 {
   struct proc_node *i;
 
-  if (linktype==DLT_EN10MB && (data[12]!=8 || data[13]!=0))
-	  return; //not an IP ethernet frame
+  if (linktype == DLT_EN10MB && (data[12] != 8 || data[13] != 0))
+    return;			//not an IP ethernet frame
+
   for (i = ip_frag_procs; i; i = i->next)
     (i->item) (data + linkoffset, hdr->len - linkoffset);
 }
@@ -144,8 +155,10 @@ gen_ip_frag_proc (u_char * data, int len)
 {
   struct proc_node *i;
   struct ip *iph = (struct ip *) data;
-  int need_free=0;
+  int need_free = 0;
   int skblen;
+  if (!nids_params.ip_filter (iph))
+    return;
   if (len < sizeof (struct ip) || iph->ip_hl < 5 || iph->ip_v != 4 ||
       ip_fast_csum ((unsigned char *) iph, iph->ip_hl) != 0
       || len < ntohs (iph->ip_len) ||
@@ -184,7 +197,7 @@ gen_ip_frag_proc (u_char * data, int len)
     free (iph);
 }
 
-static void 
+static void
 process_udp (char *x)
 {
 }
@@ -200,6 +213,8 @@ gen_ip_proc (u_char * data, int skblen)
     case IPPROTO_UDP:
       process_udp (data);
       break;
+    case IPPROTO_ICMP:
+      process_icmp (data);
     default:;
     }
 }
@@ -235,62 +250,98 @@ nids_register_ip_frag (void (*x))
   ip_procs = ipp;
 }
 
-static pcap_t *desc;
+static pcap_t *desc = 0;
 
-void
+int
 nids_init ()
 {
   char *device = nids_params.device;
   if (!device)
-    device = pcap_lookupdev (errbuf);
+    device = pcap_lookupdev (nids_errbuf);
   if (!device)
-    {
-      fprintf (stderr, "%s", errbuf);
-      exit (1);
-    }
-  nids_params.device=device;
+    return 0;
+  nids_params.device = device;
 
-  desc = pcap_open_live (device, 16384, 1, 1024, errbuf);
-//desc=pcap_open_offline("savefile",errbuf);
+  desc = pcap_open_live (device, 16384, 1, 1024, nids_errbuf);
+//desc=pcap_open_offline("savefile",nids_errbuf);
   if (!desc)
+    return 0;
+  switch (linktype = pcap_datalink (desc))
     {
-      fprintf (stderr, "%s", errbuf);
-      exit (1);
+    case DLT_EN10MB:
+      linkoffset = 14;
+      break;
+    case DLT_PPP:
+      linkoffset = 4;
+      break;
+    case DLT_RAW:
+      linkoffset = 0;
+      break;
+    default:
+      fprintf (stderr, "link type unknown, defaulting to ethernet.\n");
+      linktype = DLT_EN10MB;
+      linkoffset = 14;
+      break;
+    }
+  if (nids_params.dev_addon == -1)
+    {
+      if (linktype == DLT_EN10MB)
+	nids_params.dev_addon = 16;
+      else
+	nids_params.dev_addon = 0;
     }
-  switch(linktype=pcap_datalink(desc)) {
-  case DLT_EN10MB:
-    linkoffset = 14;
-    break;
-  case DLT_PPP:
-    linkoffset = 4;
-    break;
-  case DLT_RAW:
-    linkoffset = 0;
-    break;  
-  default:
-    fprintf(stderr, "link type unknown, defaulting to ethernet.\n");
-    linktype = DLT_EN10MB;
-    linkoffset = 14;
-    break;
-  }
-  if (nids_params.dev_addon == -1) {
-    if (linktype==DLT_EN10MB) 
-          nids_params.dev_addon = 16;
-    else 
-      nids_params.dev_addon = 0;
-  }
   if (nids_params.syslog == nids_syslog)
     openlog ("libnids", 0, LOG_LOCAL0);
   init_procs ();
   tcp_init (nids_params.n_tcp_streams);
   ip_frag_init (nids_params.n_hosts);
   scan_init ();
+  return 1;
 }
 
 void
 nids_run ()
 {
-    if (pcap_loop (desc, -1, (pcap_handler) pcap_hand, 0) <= 0)
-	pcap_perror (desc, "loop");
-    pcap_close (desc);
+  if (!desc)
+    {
+      strcpy (nids_errbuf, "Libnids not initialized");
+      return;
+    }
+  pcap_loop (desc, -1, (pcap_handler) pcap_hand, 0);
+  strcpy (nids_errbuf, "loop:");
+  strncpy (nids_errbuf + 5, pcap_geterr (desc), sizeof (nids_errbuf - 7));
+  nids_errbuf[sizeof (nids_errbuf) - 1] = 0;
+  pcap_close (desc);
+}
+
+int
+nids_getfd ()
+{
+  if (!desc)
+    {
+      strcpy (nids_errbuf, "Libnids not initialized");
+      return -1;
+    }
+  return pcap_fileno (desc);
+}
+
+int
+nids_next ()
+{
+  struct pcap_pkthdr h;
+  char *data;
+  if (!desc)
+    {
+      strcpy (nids_errbuf, "Libnids not initialized");
+      return 0;
+    }
+  if (!(data = (char *) pcap_next (desc, &h)))
+    {
+      strcpy (nids_errbuf, "next:");
+      strncpy (nids_errbuf + 5, pcap_geterr (desc), sizeof (nids_errbuf - 7));
+      nids_errbuf[sizeof (nids_errbuf) - 1] = 0;
+      return 0;
+    }
+  pcap_hand (0, &h, data);
+  return 1;
 }
