@@ -1,5 +1,5 @@
 /*
-Copyright (c) 1999 Rafal Wojtczuk <nergal@icm.edu.pl>. All rights reserved.
+Copyright (c) 1999 Rafal Wojtczuk <nergal@avet.com.pl>. All rights reserved.
 See the file COPYING for license details.
 */
 
@@ -9,8 +9,6 @@ See the file COPYING for license details.
 #include "util.h"
 #include "tcp.h"
 #endif
-void nids_init();
-void nids_run();
 struct tuple4
 {
 unsigned short source,dest;
@@ -56,15 +54,21 @@ int syslog_level;
 int scan_num_hosts;
 int scan_delay;
 int scan_num_ports;
+void (*no_mem)(char*);
+int (*ip_filter)();
 };
 extern struct nids_prm nids_params;
 
+int nids_init();
+void nids_run();
 void nids_register_tcp(void(*));
 void nids_register_ip(void(*));
 void nids_register_ip_frag(void(*));
 void nids_killtcp(struct tcp_stream*);
 void nids_discard (struct tcp_stream *, int);
+extern int nids_getfd();
 extern char *nids_warnings[];
+extern char nids_errbuf[];
 
 enum {
 NIDS_WARN_IP=1,
