@@ -1,7 +1,7 @@
 /*
-Copyright (c) 1999 Rafal Wojtczuk <nergal@icm.edu.pl>. All rights reserved.
-See the file COPYING for license details.
-*/
+   Copyright (c) 1999 Rafal Wojtczuk <nergal@avet.com.pl>. All rights reserved.
+   See the file COPYING for license details.
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -11,24 +11,30 @@ See the file COPYING for license details.
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
+#include <netinet/ip_icmp.h>
 #include "util.h"
 #include "nids.h"
 
 #ifndef __linux__
 enum
-{
-  TCP_ESTABLISHED = 1,
-  TCP_SYN_SENT,
-  TCP_SYN_RECV,
-  TCP_FIN_WAIT1,
-  TCP_FIN_WAIT2,
-  TCP_TIME_WAIT,
-  TCP_CLOSE,
-  TCP_CLOSE_WAIT,
-  TCP_LAST_ACK,
-  TCP_LISTEN,
-  TCP_CLOSING   /* now a valid state */
-};
+  {
+    TCP_ESTABLISHED = 1,
+    TCP_SYN_SENT,
+    TCP_SYN_RECV,
+    TCP_FIN_WAIT1,
+    TCP_FIN_WAIT2,
+    TCP_TIME_WAIT,
+    TCP_CLOSE,
+    TCP_CLOSE_WAIT,
+    TCP_LAST_ACK,
+    TCP_LISTEN,
+    TCP_CLOSING			/* now a valid state */
+  };
+#define ICMP_DEST_UNREACH ICMP_UNREACH
+#define ICMP_PROT_UNREACH ICMP_UNREACH_PROTOCOL
+#define ICMP_PORT_UNREACH ICMP_UNREACH_PORT
+#define NR_ICMP_UNREACH   ICMP_MAXTYPE
+
 #endif /* !linux */
 
 extern struct proc_node *tcp_procs;
@@ -48,6 +54,7 @@ static void
 free_tcp (struct tcp_stream *a_tcp)
 {
   int hash_index = a_tcp->hash_index;
+  struct lurker_node *i, *j;
   if (a_tcp->next_node)
     a_tcp->next_node->prev_node = a_tcp->prev_node;
   if (a_tcp->prev_node)
@@ -66,18 +73,55 @@ free_tcp (struct tcp_stream *a_tcp)
     tcp_oldest = a_tcp->prev_time;
   if (a_tcp == tcp_latest)
     tcp_latest = a_tcp->next_time;
+
+  i = a_tcp->listeners;
+
+  while (i)
+    {
+      j = i->next;
+      free (i);
+      i = j;
+    }
+
 //  free (a_tcp);
   a_tcp->next_free = free_streams;
   free_streams = a_tcp;
   tcp_num--;
 }
 
+static int
+mk_hash_index (struct tuple4 addr)
+{
+  unsigned int sth = (addr.dest << 16) + addr.source;
+  unsigned int hash = 0;
+  sth ^= addr.saddr;
+  sth += addr.daddr;
+  do
+    {				// taken from scanlogd.c
+
+      hash ^= sth;
+    }
+  while ((sth >>= hash_deg));
+  hash %= tcp_stream_table_size;
+  return hash;
+}
+
+
 
 static void
-add_new_tcp (int hash_index, struct tuple4 addr, struct tcphdr *this_tcphdr)
+add_new_tcp (struct tcphdr *this_tcphdr, struct ip *this_iphdr)
 {
   struct tcp_stream *tolink;
   struct tcp_stream *a_tcp;
+  int hash_index;
+  struct tuple4 addr;
+
+  addr.source = ntohs (this_tcphdr->th_sport);
+  addr.dest = ntohs (this_tcphdr->th_dport);
+  addr.saddr = this_iphdr->ip_src.s_addr;
+  addr.daddr = this_iphdr->ip_dst.s_addr;
+  hash_index = mk_hash_index (addr);
+
   if (tcp_num > max_stream)
     {
       struct lurker_node *i;
@@ -119,26 +163,6 @@ add_new_tcp (int hash_index, struct tuple4 addr, struct tcphdr *this_tcphdr)
   tcp_latest = a_tcp;
 }
 
-#ifdef HASH_DEBUG
-static char * hash_param1="wC30Uff.iIc3Q";
-static char * hash_param2="UEIeBIMUOb9sg";
-#endif
-
-static int
-mk_hash_index (struct tuple4 addr)
-{
-  unsigned int sth = (addr.dest << 16) + addr.source;
-  unsigned int hash = 0;
-  sth ^= addr.saddr;
-  sth += addr.daddr;
-  do
-    {				// taken from scanlogd.c
-      hash ^= sth;
-    }
-  while ((sth >>= hash_deg));
-  hash %= tcp_stream_table_size;
-  return hash;
-}
 
 static void
 add2buf (struct half_stream *rcv, char *data, int datalen)
@@ -161,7 +185,7 @@ add2buf (struct half_stream *rcv, char *data, int datalen)
 	  rcv->bufsize *= 2;
 	}
       if (!rcv->data)
-	mem_low ();
+	nids_params.no_mem ("add2buf");
     }
   memcpy (rcv->data + rcv->count - rcv->offset, data, datalen);
   rcv->count_new = datalen;
@@ -178,12 +202,48 @@ add2buf (struct half_stream *rcv, char *data, int datalen)
 
 #define EXP_SEQ (snd->first_data_seq+rcv->count+rcv->urg_count)
 
+static void
+ride_lurkers (struct tcp_stream *a_tcp, char mask)
+{
+  struct lurker_node *i;
+  char cc, sc, ccu, scu;
+
+  for (i = a_tcp->listeners; i; i = i->next)
+    if (i->whatto & mask)
+      {
+	cc = a_tcp->client.collect;
+	sc = a_tcp->server.collect;
+	ccu = a_tcp->client.collect_urg;
+	scu = a_tcp->server.collect_urg;
+
+	(i->item) (a_tcp, &i->data);
+	if (cc < a_tcp->client.collect)
+	  i->whatto |= COLLECT_cc;
+	if (ccu < a_tcp->client.collect_urg)
+	  i->whatto |= COLLECT_ccu;
+	if (sc < a_tcp->server.collect)
+	  i->whatto |= COLLECT_sc;
+	if (scu < a_tcp->server.collect_urg)
+	  i->whatto |= COLLECT_scu;
+	if (cc > a_tcp->client.collect)
+	  i->whatto &= ~COLLECT_cc;
+	if (ccu > a_tcp->client.collect_urg)
+	  i->whatto &= ~COLLECT_ccu;
+	if (sc > a_tcp->server.collect)
+	  i->whatto &= ~COLLECT_sc;
+	if (scu > a_tcp->server.collect_urg)
+	  i->whatto &= ~COLLECT_scu;
+
+      }
+}
+
 static void
 notify (struct tcp_stream *a_tcp, struct half_stream *rcv,
 	struct half_stream *snd)
 {
-  struct lurker_node *i;
+  struct lurker_node *i,**prev_addr;
   char mask;
+
   if (rcv->count_new_urg)
     {
       if (!rcv->collect_urg)
@@ -192,10 +252,8 @@ notify (struct tcp_stream *a_tcp, struct half_stream *rcv,
 	mask = COLLECT_ccu;
       else
 	mask = COLLECT_scu;
-      for (i = a_tcp->listeners; i; i = i->next)
-	if (i->whatto & mask)
-	  (i->item) (a_tcp, &i->data);
-      return;
+      ride_lurkers (a_tcp, mask);
+      goto prune_listeners;
     }
   a_tcp->read = rcv->count - rcv->offset;
   if (rcv->collect)
@@ -205,9 +263,7 @@ notify (struct tcp_stream *a_tcp, struct half_stream *rcv,
       else
 	mask = COLLECT_sc;
 
-      for (i = a_tcp->listeners; i; i = i->next)
-	if (i->whatto & mask)
-	  (i->item) (a_tcp, &i->data);
+      ride_lurkers (a_tcp, mask);
 
       rcv->count_new = 0;
       if (a_tcp->read > 0)
@@ -216,6 +272,21 @@ notify (struct tcp_stream *a_tcp, struct half_stream *rcv,
 	  rcv->offset += a_tcp->read;
 	}
     }
+prune_listeners:
+  prev_addr = &a_tcp->listeners;
+  i = a_tcp->listeners;
+  while (i)
+    if (!i->whatto)
+      {
+	*prev_addr = i->next;
+	free (i);
+	i = *prev_addr;
+      }
+    else
+      {
+	prev_addr = &i->next;
+	i = i->next;
+      }
 }
 
 
@@ -238,45 +309,51 @@ add_from_skb (struct tcp_stream *a_tcp, struct half_stream *rcv,
       before (rcv->urg_ptr, this_seq + datalen))
     {
       to_copy = rcv->urg_ptr - (this_seq + lost);
-      if (to_copy > 0) {
-	if (rcv->collect)
-	  {
-	    add2buf (rcv, data + lost, to_copy);
-	    notify (a_tcp, rcv, snd);
-	  }
-	else {
-	  rcv->count += to_copy;
+      if (to_copy > 0)
+	{
+	  if (rcv->collect)
+	    {
+	      add2buf (rcv, data + lost, to_copy);
+	      notify (a_tcp, rcv, snd);
+	    }
+	  else
+	    {
+	      rcv->count += to_copy;
+	    }
 	}
-      }
       rcv->urgdata = data[rcv->urg_ptr - this_seq];
       rcv->count_new_urg = 1;
       notify (a_tcp, rcv, snd);
       rcv->count_new_urg = 0;
       rcv->urg_count++;
       to_copy2 = this_seq + datalen - rcv->urg_ptr - 1;
-      if (to_copy2 > 0) {
-	if (rcv->collect)
-	  {
-	    add2buf (rcv, data + lost + to_copy + 1, to_copy2);
-	    notify (a_tcp, rcv, snd);
-	  }
-	else {
-	  rcv->count += to_copy2;
+      if (to_copy2 > 0)
+	{
+	  if (rcv->collect)
+	    {
+	      add2buf (rcv, data + lost + to_copy + 1, to_copy2);
+	      notify (a_tcp, rcv, snd);
+	    }
+	  else
+	    {
+	      rcv->count += to_copy2;
+	    }
 	}
-      }
     }
   else
     {
-      if (datalen > 0) {
-	if (rcv->collect)
-	  {
-	    add2buf (rcv, data + lost, datalen - lost);
-	    notify (a_tcp, rcv, snd);
-	  }
-	else {
-	  rcv->count += datalen - lost;
+      if (datalen > 0)
+	{
+	  if (rcv->collect)
+	    {
+	      add2buf (rcv, data + lost, datalen - lost);
+	      notify (a_tcp, rcv, snd);
+	    }
+	  else
+	    {
+	      rcv->count += datalen - lost;
+	    }
 	}
-      }
     }
   if (fin)
     snd->state = FIN_SENT;
@@ -301,7 +378,7 @@ tcp_queue (struct tcp_stream *a_tcp, struct tcphdr *this_tcphdr,
 	{
 	  /* the packet straddles our window end */
 	  add_from_skb (a_tcp, rcv, snd, data, datalen, this_seq,
-			(this_tcphdr->th_flags & TH_FIN), (this_tcphdr->th_flags & TH_URG),
+	  (this_tcphdr->th_flags & TH_FIN), (this_tcphdr->th_flags & TH_URG),
 			ntohs (this_tcphdr->th_urp) + this_seq - 1);
 	  /*
 	   * Do we have any old packets to ack that the above
@@ -353,7 +430,7 @@ tcp_queue (struct tcp_stream *a_tcp, struct tcphdr *this_tcphdr,
       pakiet->len = datalen;
       pakiet->data = malloc (datalen);
       if (!pakiet->data)
-	mem_low ();
+	nids_params.no_mem ("tcp_queue");
       memcpy (pakiet->data, data, datalen);
       pakiet->fin = (this_tcphdr->th_flags & TH_FIN);
       pakiet->seq = this_seq;
@@ -432,13 +509,51 @@ check_flags (struct ip *iph, struct tcphdr *th)
 
 extern void detect_scan (struct ip *);
 
+struct tcp_stream *
+find_stream (struct tcphdr *this_tcphdr, struct ip *this_iphdr,
+	     int *from_client)
+{
+  struct tuple4 this_addr, reversed;
+  int hash_index;
+  struct tcp_stream *a_tcp;
+  this_addr.source = ntohs (this_tcphdr->th_sport);
+  this_addr.dest = ntohs (this_tcphdr->th_dport);
+  this_addr.saddr = this_iphdr->ip_src.s_addr;
+  this_addr.daddr = this_iphdr->ip_dst.s_addr;
+  hash_index = mk_hash_index (this_addr);
+  for (a_tcp = tcp_stream_table[hash_index];
+       a_tcp && !b_comp (a_tcp->addr, this_addr);
+       a_tcp = a_tcp->next_node);
+  if (a_tcp)
+    {
+      *from_client = 1;
+      return a_tcp;
+    }
+  reversed.source = ntohs (this_tcphdr->th_dport);
+  reversed.dest = ntohs (this_tcphdr->th_sport);
+  reversed.saddr = this_iphdr->ip_dst.s_addr;
+  reversed.daddr = this_iphdr->ip_src.s_addr;
+  hash_index = mk_hash_index (reversed);
+  for (a_tcp = tcp_stream_table[hash_index];
+       a_tcp && !b_comp (a_tcp->addr, reversed);
+       a_tcp = a_tcp->next_node);
+  if (a_tcp)
+    {
+      *from_client = 0;
+      return a_tcp;
+    }
+  else
+    return 0;
+}
+
+
+
 void
 process_tcp (u_char * data, int skblen)
 {
-  struct tuple4 this_addr, reversed;
   struct ip *this_iphdr = (struct ip *) data;
   struct tcphdr *this_tcphdr = (struct tcphdr *) (data + 4 * this_iphdr->ip_hl);
-  int hash_index, hash_index_rev, datalen, iplen;
+  int datalen, iplen;
   int from_client = 1;
   struct tcp_stream *a_tcp;
   struct half_stream *snd, *rcv;
@@ -465,31 +580,11 @@ process_tcp (u_char * data, int skblen)
   if (!(this_tcphdr->th_flags & TH_ACK))
     detect_scan (this_iphdr);
   check_flags (this_iphdr, this_tcphdr);
-  this_addr.source = ntohs (this_tcphdr->th_sport);
-  this_addr.dest = ntohs (this_tcphdr->th_dport);
-  this_addr.saddr = this_iphdr->ip_src.s_addr;
-  this_addr.daddr = this_iphdr->ip_dst.s_addr;
-  hash_index = mk_hash_index (this_addr);
-  for (a_tcp = tcp_stream_table[hash_index];
-       a_tcp && !b_comp (a_tcp->addr, this_addr);
-       a_tcp = a_tcp->next_node);
-  if (!a_tcp)
-    {
-      reversed.source = ntohs (this_tcphdr->th_dport);
-      reversed.dest = ntohs (this_tcphdr->th_sport);
-      reversed.saddr = this_iphdr->ip_dst.s_addr;
-      reversed.daddr = this_iphdr->ip_src.s_addr;
-      hash_index_rev = mk_hash_index (reversed);
-      from_client = 0;
-      for (a_tcp = tcp_stream_table[hash_index_rev];
-	   a_tcp && !b_comp (a_tcp->addr, reversed);
-	   a_tcp = a_tcp->next_node);
-    }
-  if (!a_tcp)
+  if (!(a_tcp = find_stream (this_tcphdr, this_iphdr, &from_client)))
     {
       if ((this_tcphdr->th_flags & TH_SYN) && !(this_tcphdr->th_flags & TH_ACK) &&
 	  !(this_tcphdr->th_flags & TH_RST))
-	add_new_tcp (hash_index, this_addr, this_tcphdr);
+	add_new_tcp (this_tcphdr, this_iphdr);
       return;
     }
   if (from_client)
@@ -506,7 +601,7 @@ process_tcp (u_char * data, int skblen)
   if ((this_tcphdr->th_flags & TH_SYN))
     {
       if (from_client || a_tcp->client.state != TCP_SYN_SENT ||
-	  a_tcp->server.state != TCP_CLOSE || !(this_tcphdr->th_flags & TH_ACK))
+      a_tcp->server.state != TCP_CLOSE || !(this_tcphdr->th_flags & TH_ACK))
 	return;
       if (a_tcp->client.seq != ntohl (this_tcphdr->th_ack))
 	return;
@@ -623,6 +718,8 @@ process_tcp (u_char * data, int skblen)
   snd->window = ntohs (this_tcphdr->th_win);
   if (rcv->rmem_alloc > 65535)
     prune_queue (rcv, this_tcphdr);
+  if (!a_tcp->listeners)
+    free_tcp (a_tcp);
 }
 
 void
@@ -650,6 +747,8 @@ tcp_init (int size)
 
   tcp_stream_table_size = nids_params.n_tcp_streams;
   tcp_stream_table = malloc (tcp_stream_table_size * sizeof (char *));
+  if (!tcp_stream_table)
+    nids_params.no_mem ("tcp_init");
   memset (tcp_stream_table, 0, tcp_stream_table_size * sizeof (char *));
   tmp = tcp_stream_table_size;
   hash_deg = 32;
@@ -661,9 +760,69 @@ tcp_init (int size)
     }
   max_stream = 3 * tcp_stream_table_size / 4;
   streams_pool = (struct tcp_stream *) malloc ((max_stream + 1) * sizeof (struct tcp_stream));
+  if (!streams_pool)
+    nids_params.no_mem ("tcp_init");
   for (i = 0; i < max_stream; i++)
     streams_pool[i].next_free = &(streams_pool[i + 1]);
   streams_pool[max_stream].next_free = 0;
   free_streams = streams_pool;
   return 0;
 }
+
+extern unsigned short ip_compute_csum (char *, int len);
+
+void
+process_icmp (u_char * data)
+{
+  struct ip *iph = (struct ip *) data;
+  struct ip *orig_ip;
+  struct icmp *pkt;
+  struct tcphdr *th;
+  struct half_stream *hlf;
+  int match_addr;
+  struct tcp_stream *a_tcp;
+  struct lurker_node *i;
+
+  int from_client;
+  int len = ntohs (iph->ip_len) - (iph->ip_hl << 2);
+
+  if (len < sizeof (struct icmp))
+      return;
+  pkt = (struct icmp *) (data + (iph->ip_hl << 2));
+  if (ip_compute_csum ((char *) pkt, len))
+    return;
+  if (pkt->icmp_type != ICMP_DEST_UNREACH)
+    return;
+  len -= 8;			// sizeof (struct icmp) is not what we want here
+
+  if (len < sizeof (struct ip))
+      return;
+  orig_ip = (struct ip *) (((char *) pkt) + 8);
+  len -= orig_ip->ip_hl << 2;
+  if (len < 8)
+    return;
+  if ((pkt->icmp_code & 15) == ICMP_PROT_UNREACH ||
+      (pkt->icmp_code & 15) == ICMP_PORT_UNREACH)
+    match_addr = 1;
+  else
+    match_addr = 0;
+  if (pkt->icmp_code > NR_ICMP_UNREACH)
+    return;
+  if (match_addr && (iph->ip_src.s_addr != orig_ip->ip_dst.s_addr))
+    return;
+  if (orig_ip->ip_p != IPPROTO_TCP)
+    return;
+  th = (struct tcphdr *) (((char *) orig_ip) + (orig_ip->ip_hl << 2));
+  if (!(a_tcp = find_stream (th, orig_ip, &from_client)))
+    return;
+  if (a_tcp->addr.dest == iph->ip_dst.s_addr)
+    hlf = &a_tcp->server;
+  else
+    hlf = &a_tcp->client;
+  if (hlf->state != TCP_SYN_SENT && hlf->state != TCP_SYN_RECV)
+    return;
+  a_tcp->nids_state = NIDS_RESET;
+  for (i = a_tcp->listeners; i; i = i->next)
+    (i->item) (a_tcp, &i->data);
+  free_tcp (a_tcp);
+}
