@@ -1,15 +1,18 @@
 /*
-Copyright (c) 1999 Rafal Wojtczuk <nergal@icm.edu.pl>. All rights reserved.
+Copyright (c) 1999 Rafal Wojtczuk <nergal@avet.com.pl>. All rights reserved.
 See the file COPYING for license details.
 */
 
-#include "util.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include "util.h"
+#include "nids.h"
+
+
 void 
-mem_low ()
+nids_no_mem (char * func)
 {
-  fprintf (stderr, "Out of memory.\n");
+  fprintf (stderr, "Out of memory in %s.\n",func);
   exit (1);
 }
 
@@ -18,7 +21,7 @@ test_malloc (int x)
 {
   char *ret = malloc (x);
   if (!ret)
-    mem_low ();
+    nids_params.no_mem("test_malloc");
   return ret;
 }
 inline int 
